﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using AudioPrecision.API;

namespace APxControl
{
    public partial class Form1 : Form
    {
      //Create an APx object
    private APx500 APx = new APx500();

      public Form1()
        {
            CreateAPxObject();

            // This call is required by the designer.
            InitializeComponent();
        }
        private void CreateAPxObject()
        {
            APx = new AudioPrecision.API.APx500();
        }
        private void button1_Click(object sender, EventArgs e)
        {
            APx.Visible = true;
        }
        private void button2_Click(object sender, EventArgs e)
        {
            string filename = "SampleProject.approjx";
            string directory = System.Environment.CurrentDirectory;
            string fullpath = directory + "\\" + filename;

            APx.OperatingMode = APxOperatingMode.SequenceMode;
            APx.OpenProject(fullpath);
            APx.Sequence.Run();

            //Get Settled results from Level and Gain measurement.
            ISequenceMeasurement measurement = APx.Sequence["Signal Path1"]["Level and Gain"];

            //Check to ensure settled results are available
            if (measurement.HasSequenceResults == true)
            {
                //Get the object for the Level and Gain sequence results
                ISequenceResultCollection Results = default(ISequenceResultCollection);
                Results = measurement.SequenceResults;

                //get the meter readings
                //RMS Level
                double[] RMSLevelValues = Results[MeasurementResultType.LevelMeter].GetMeterValues();

                //Gain
                double[] GainValues = Results[MeasurementResultType.GainMeter].GetMeterValues();

                //Peak Level
                double[] PeakLevelValues = Results[MeasurementResultType.PeakLevelMeter].GetMeterValues();
            }

            //Get X/Y results from Stepped Frequency Sweep
            measurement = APx.Sequence["Signal Path1"]["Stepped Frequency Sweep"];

            //Check the to ensure results are available
            if (measurement.HasSequenceResults == true)
            {
                //Data Variables
                double[] RMSLevelXAxis_Data = null;
                double[] RMSLevelYAxis_Ch1_LevelData = null;
                double[] RMSLevelYAxis_Ch2_LevelData = null;

                double[] SmoothXValues = null;
                double[] SmoothYValues_Ch1 = null;
                double[] SmoothYValues_Ch2 = null;

                double[] DeviationXValues = null;

                //Get the object for the Stepped Frequency Sweep sequence results
                ISequenceResultCollection Results = default(ISequenceResultCollection);
                Results = APx.Sequence["Signal Path1"]["Stepped Frequency Sweep"].SequenceResults;

                //RMS Level
                ISequenceResult LevelVsFrequencyXYResults = default(ISequenceResult);
                //Get the object for the LevelVsFrequency results
                LevelVsFrequencyXYResults = Results[MeasurementResultType.LevelVsFrequency];

                RMSLevelXAxis_Data = LevelVsFrequencyXYResults.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1);
                RMSLevelYAxis_Ch1_LevelData = LevelVsFrequencyXYResults.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1);
                RMSLevelYAxis_Ch2_LevelData = LevelVsFrequencyXYResults.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1);

                //RMS Level Smoothed Derived Result
                ISequenceResult SmoothedXYResults = default(ISequenceResult);
                //Get the object for the RMS Level -> Smooth results
                SmoothedXYResults = Results["RMS Level -> Smooth"];

                SmoothXValues = SmoothedXYResults.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1);
                SmoothYValues_Ch1 = SmoothedXYResults.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1);
                SmoothYValues_Ch2 = SmoothedXYResults.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1);

                //Deviation
                DeviationXValues = Results["Deviation (20.0000 Hz - 20.0000 kHz)"].GetMeterValues();

                //Alternate approach using ForEach statement to get the data
                foreach (ISequenceResult Result in Results)
                {
                    //RMS Level
                    if (Result.Name == "RMS Level")
                    {
                        RMSLevelXAxis_Data = Result.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1);
                        RMSLevelYAxis_Ch1_LevelData = Result.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1);
                        RMSLevelYAxis_Ch2_LevelData = Result.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1);
                    }
                    //RMS Level -> Smooth
                    if (Result.ResultType.Equals(MeasurementResultType.Smooth) & Result.Name == "RMS Level -> Smooth")
                    {
                        SmoothXValues = Result.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1);
                        SmoothYValues_Ch1 = Result.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1);
                        SmoothYValues_Ch2 = Result.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1);
                    }
                    //Deviation
                    if (Result.Name == "Deviation (20.0000 Hz - 20.0000 kHz)")
                    {
                        DeviationXValues = Result.GetMeterValues();
                    }
                }
            }
        }
        private void button3_Click(object sender, EventArgs e)
        {
            APx.OperatingMode = APxOperatingMode.BenchMode;

            //Configure Signal Path Setup Input/Output
            APx.BenchMode.Setup.OutputConnector.Type = OutputConnectorType.AnalogBalanced;
            APx.BenchMode.Setup.AnalogOutput.ChannelCount = 2;
            APx.BenchMode.Setup.AnalogOutput.BalancedSourceImpedance = AnalogBalancedSourceImpedance.SourceImpedance_40;

            APx.BenchMode.Setup.InputConnector.Type = InputConnectorType.AnalogBalanced;
            APx.BenchMode.Setup.AnalogInput.ChannelCount = 2;
            APx.BenchMode.Setup.AnalogInput.SetTermination(InputChannelIndex.Ch1, AnalogInputTermination.InputTermination_Bal_200k);
            APx.BenchMode.Setup.AnalogInput.UseHighPerformanceSineAnalyzer = true;

            //Configure Signal Path Setup Filters
            APx.BenchMode.Setup.HighpassFilter = HighpassFilterMode.DC;
            APx.BenchMode.Setup.LowpassFilterAnalog = LowpassFilterModeAnalog.Elliptic;
            APx.BenchMode.Setup.LowpassFilterFrequencyAnalog = 20000;
            APx.BenchMode.Setup.LowpassFilterAnalogBandwidth = AdcBandwidth.bw90k;
            APx.BenchMode.Setup.WeightingFilter = SignalPathWeightingFilterType.wt_None;
            APx.BenchMode.Setup.DeviceDelay.Value = 0;

            //Configure Generator
            APx.BenchMode.Generator.AutoOn = false;
            APx.BenchMode.Generator.AnalogSineMode = AnalogSineGeneratorMode.HighPerformanceSineGenerator;
            APx.BenchMode.Generator.Levels.SetValue(OutputChannelIndex.Ch1, 1.0);
            APx.BenchMode.Generator.Frequency.Value = 2000;
            APx.BenchMode.Generator.On = true;

            //Configure Analyzer
            APx.BenchMode.Analyzer.BandpassFilterTuning = FilterTuningType.GeneratorFrequency;
            APx.BenchMode.Analyzer.ThdNWeighting = SignalPathWeightingFilterType.wt_None;
            APx.BenchMode.Analyzer.BandpassFilterTuning = FilterTuningType.GeneratorFrequency;
            APx.BenchMode.Analyzer.ReferenceChannel = InputChannelIndex.Ch1;
            APx.BenchMode.Analyzer.ImdMeterType = ImdAnalysisType.SmpteDin;

            //Monitors/Meters
            //Add required Monitors/Meters
            APx.BenchMode.Meters.Show();
            APx.BenchMode.Meters.Enabled = false;

            //Delete current Monitors/Meters  
            int MeterCount = APx.BenchMode.Meters.Count;
            int meterIndex = 0;
            for (meterIndex = MeterCount - 1; meterIndex >= 0; meterIndex += -1)
            {
                APx.BenchMode.Meters.Remove(meterIndex);
            }

            //Add desired Monitors/Meters
            APx.BenchMode.Meters.Add(BenchModeMeterType.BandpassLevelMeter);
            APx.BenchMode.Meters.Add(BenchModeMeterType.DcLevelMeter);
            APx.BenchMode.Meters.Add(BenchModeMeterType.GainMeter);
            APx.BenchMode.Meters.Add(BenchModeMeterType.ThdNRatioMeter);

            //Configure settling For desired Monitors/Meters
            APx.BenchMode.Analyzer.Settling.Enabled = true;
            APx.BenchMode.Analyzer.Settling.ReadingTimeout = 5;
            APx.BenchMode.Analyzer.Settling.TrackFirst = true;

            APx.BenchMode.Analyzer.Settling[SettlingMeterType.RmsLevel].ResetToDefaults();

            APx.BenchMode.Analyzer.Settling[SettlingMeterType.BandpassLevel][InputChannelIndex.Ch1].Algorithm = SettlingAlgorithmEx.Average;

            ISettlingParametersEx ChannelSettlingParameters = default(ISettlingParametersEx);
            ChannelSettlingParameters = APx.BenchMode.Analyzer.Settling[SettlingMeterType.BandpassLevel][InputChannelIndex.Ch1];
            ChannelSettlingParameters.Algorithm = SettlingAlgorithmEx.None;
            ChannelSettlingParameters.DelayTime = 0.031;
            ChannelSettlingParameters.DelayCycles = 4;
            ChannelSettlingParameters.SettlingTime = 0.251;
            ChannelSettlingParameters.SettlingCycles = 4;
            ChannelSettlingParameters.Points = 4;
            ChannelSettlingParameters.Tolerance = 4;
            ChannelSettlingParameters.Floor = 2E-08;

            APx.BenchMode.Analyzer.Settling.ResetToDefaults();

            APx.BenchMode.Meters.Enabled = true;

            //Get Settled readings from meters all at the same time.
            //Create an array of settling meter types
            SettlingMeterType[] SettledMeterTypes = {SettlingMeterType.BandpassLevel, SettlingMeterType.DcLevel, SettlingMeterType.Gain, SettlingMeterType.ThdNRatio};
            //Pass the array of meter types to get the ISettledResultCollection 
            ISettledResultCollection MeterReadings = default(ISettledResultCollection);
            MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettledMeterTypes);

            //Get readings for each meter type
            double[] BandpassValues = MeterReadings[SettlingMeterType.BandpassLevel].GetValues("Vrms");
            double[] DCLevelValues = MeterReadings[SettlingMeterType.DcLevel].GetValues("V");
            double[] GainValues = MeterReadings[SettlingMeterType.Gain].GetValues("dB");
            double[] ThdNValues = MeterReadings[SettlingMeterType.ThdNRatio].GetValues("dB");

            //Get Settled readings from meters one at a time.
            MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettlingMeterType.BandpassLevel);
            BandpassValues = MeterReadings[0].GetValues("Vrms");

            MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettlingMeterType.DcLevel);
            DCLevelValues = MeterReadings[0].GetValues("V");

            MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettlingMeterType.Gain);
            GainValues = MeterReadings[0].GetValues("dB");

            MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettlingMeterType.ThdNRatio);
            ThdNValues = MeterReadings[0].GetValues("dB");

            //Sweep measurement
            //Configure measurement settings
            APx.BenchMode.Measurements.SteppedSweep.Show();
            APx.BenchMode.Measurements.SteppedSweep.Repeat = false;
            APx.BenchMode.Measurements.SteppedSweep.Append = false;
            APx.BenchMode.Measurements.SteppedSweep.Source = SweepSourceParameterType.GeneratorFrequency;
            APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Start.Unit = "Hz";
            APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Start.Value = 20;
            APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Stop.Value = 20000;
            APx.BenchMode.Measurements.SteppedSweep.SourceParameters.StepType = SweepStepType.Log;
            APx.BenchMode.Measurements.SteppedSweep.SourceParameters.NumberOfPoints = 31;

            //Set Right axis data To THD+N Ratio
            APx.BenchMode.SignalPathSettingsVisible = false;
            APx.BenchMode.GeneratorAnalyzerSettingsVisible = false;

            APx.BenchMode.Measurements.SteppedSweep.Graphs[0].Result.AsXYGraph().DefineRightAxisResult(MeasurementResultType.ThdNRatioVsFrequency);

            APx.BenchMode.Measurements.SteppedSweep.ReadingRate = SweepReadingRate.ReadingRate_32;
            APx.BenchMode.Measurements.SteppedSweep.PreSweepDelay = 0.3;

            APx.BenchMode.Measurements.SteppedSweep.NestedSweep.Source = NestedSweepParameterType.GeneratorLevel;
            APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.Start.Value = 1;
            APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.Stop.Value = 2;
            APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.StepType = SweepStepType.Lin;
            APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.NumberOfPoints = 2;

            APx.BenchMode.Measurements.SteppedSweep.Start();

            //Wait for Sweep to finish
            while (APx.BenchMode.Measurements.SteppedSweep.IsStarted == true)
            {
                System.Threading.Thread.Sleep(1000);
            }

            //Get Sweep data from the two Sweeps
            IXYGraph SweepXYResults = default(IXYGraph);
            SweepXYResults = APx.BenchMode.Measurements.SteppedSweep.Graphs["RMS Level"].Result.AsXYGraph();

            double[] XAxisValues = null;
            double[] Acq1YAxis_LevelValues = null;
            double[] Acq1YAxis_THDNValues = null;

            double[] Acq2YAxis_LevelValues = null;
            double[] Acq2YAxis_THDNValues = null;

            //Get X values from Sweep 1
            XAxisValues = SweepXYResults.GetXValues(InputChannelIndex.Ch1, 0, "Hz", VerticalAxis.Left);

            //Get Y values from Sweep 1
            Acq1YAxis_LevelValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 0, "Vrms", VerticalAxis.Left);
            Acq1YAxis_THDNValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 0, "dB", VerticalAxis.Right);
            //Get Y values from Sweep 2
            Acq2YAxis_LevelValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 1, "Vrms", VerticalAxis.Left);
            Acq2YAxis_THDNValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 1, "dB", VerticalAxis.Right);

            //FFT measurement
            //Configure measurement settings
            APx.BenchMode.Measurements.Fft.Show();
            APx.BenchMode.Measurements.Fft.Repeat = false;
            APx.BenchMode.Measurements.Fft.Append = false;
            APx.BenchMode.Measurements.Fft.AcquisitionType = AcqLengthType.Auto;
            APx.BenchMode.Measurements.Fft.FFTLength = FFTLength.FFT_96000;
            APx.BenchMode.Measurements.Fft.WindowType = WindowType.BlackmanHarris;
            APx.BenchMode.Measurements.Fft.TriggerType = TriggerType.PositiveZeroCrossing;
            APx.BenchMode.Measurements.Fft.TriggerChannel = InputChannelIndex.Ch1;
            APx.BenchMode.Measurements.Fft.AveragingType = AveragingType.Power;
            APx.BenchMode.Measurements.Fft.Averages = 8;
            APx.BenchMode.Measurements.Fft.DelayTimeInSeconds = 0.1;

            APx.BenchMode.Measurements.Fft.Start();

            //Wait For FFT To finish
            while (APx.BenchMode.Measurements.Fft.IsStarted == true)
            {
                System.Threading.Thread.Sleep(1000);
            }

            //Get all FFT data
            IXYGraph FFTXYResults = default(IXYGraph);
            FFTXYResults = APx.BenchMode.Measurements.Fft.Graphs["FFT Spectrum"].Result.AsXYGraph();

            double[] FFTXAxisValues = null;
            double[] FFTYAxis_Ch1_LevelValues = null;
            double[] FFTYAxis_Ch2_LevelValues = null;

            FFTXAxisValues = FFTXYResults.GetAllXValues(InputChannelIndex.Ch1, "Hz", VerticalAxis.Left);
            FFTYAxis_Ch1_LevelValues = FFTXYResults.GetAllYValues(InputChannelIndex.Ch1, "Vrms", VerticalAxis.Left);
            FFTYAxis_Ch2_LevelValues = FFTXYResults.GetAllYValues(InputChannelIndex.Ch2, "Vrms", VerticalAxis.Left);

            //Get all Scope data
            IXYGraph ScopeXYResults = default(IXYGraph);
            ScopeXYResults = APx.BenchMode.Measurements.Fft.Graphs["Scope"].Result.AsXYGraph();

            double[] ScopeXAxisValues = null;
            double[] ScopeYAxis_Ch1_LevelValues = null;
            double[] ScopeYAxis_Ch2_LevelValues = null;

            ScopeXAxisValues = ScopeXYResults.GetAllXValues(InputChannelIndex.Ch1, "s", VerticalAxis.Left);
            ScopeYAxis_Ch1_LevelValues = ScopeXYResults.GetAllYValues(InputChannelIndex.Ch1, "V", VerticalAxis.Left);
            ScopeYAxis_Ch2_LevelValues = ScopeXYResults.GetAllYValues(InputChannelIndex.Ch2, "V", VerticalAxis.Left);
        }
    }
}
