import sys
import clr 											                        # Import the Common Language Runtime
import time

sys.path.append(r'C:\Program Files\IronPython 2.7\Lib')                     # Set the path to import pure Python Libs
sys.path.append(r"C:\Program Files (x86)\Audio Precision\APx500 4.4\API")   # Set the path to import the Apx API
clr.AddReference("System.Drawing") 					                        # Needed for Dialog Boxes
clr.AddReference("System.Windows.Forms") 			                        # Needed for Dialog Boxes
clr.AddReference("AudioPrecision.API.dll", "AudioPrecision.API2.dll") 	    # Adding References to the APx API
clr.AddReference("System.Collections")

# Import the APx API
from AudioPrecision.API import *

from System.Drawing import Point # Needed for Dialog Boxes
from System.Windows.Forms import Application, Button, Form, Label # Needed for Dialog Boxes
from System.IO import Directory, Path # Used to point to the sample project file
from System import Array

class Container(Form):

    def __init__(self):
        self.Text = 'APx IronPython Example'
        self.Height = 155
        self.Width = 320
        
        bLoad = Button()
        bLoad.Text = "Load APx Software"
        bLoad.Location = Point(20, 15)
        bLoad.Width = 265
        bLoad.Click += self.APxLoad

        bProj = Button()
        bProj.Text = "Sequence Mode Automation"
        bProj.Location = Point(20, 45)
        bProj.Width = 265
        bProj.Click += self.APxSequenceMode

        bMeas = Button()
        bMeas.Text = "Bench Mode Automation"
        bMeas.Location = Point(20, 75)
        bMeas.Width = 265
        bMeas.Click += self.APxBenchMode

        self.Controls.Add(bLoad)
        self.Controls.Add(bProj)
        self.Controls.Add(bMeas)

    def APxLoad(self, sender, args):
        print "Load APx Application software"
        
        APx = APx500_Application()
        APx.Visible = True
    
    def APxSequenceMode(self, sender, args):
        print "Sequence Mode Automation"
        
        filename = "SampleProject.approjx"
        directory = Directory.GetCurrentDirectory()
        fullpath = Path.Combine(directory, filename)
        
        APx = APx500_Application(APxOperatingMode.SequenceMode)
        APx.OpenProject(fullpath)
        APx.Sequence.Run()
        
        #Get Settled results from Level and Gain measurement
        measurement = APx.Sequence["Signal Path1"]["Level and Gain"]
		
        #Check to ensure settled results are available        
        HasResults = measurement.HasSequenceResults
        if HasResults:
            #Get the object for the Level and Gain Sequence Results
            Results = measurement.SequenceResults
            
            #Get the meter readings
            #RMS Level
            RMSLevelValues = Results[MeasurementResultType.LevelMeter].GetMeterValues()
            print RMSLevelValues
           
            #Gain
            GainValues = Results[MeasurementResultType.GainMeter].GetMeterValues()
            print GainValues
            
            #Peak Level
            PeakLevelValues = Results[MeasurementResultType.PeakLevelMeter].GetMeterValues()
            print PeakLevelValues
            
        #Get X/Y values from Stepped Frequency Sweep
        measurement = APx.Sequence["Signal Path1"]["Stepped Frequency Sweep"]
        
        #Check to ensure results are available
        HasResults = measurement.HasSequenceResults
        if HasResults:
            #Get the object for the Stepped Frequency Sweep sequence results
            Results = measurement.SequenceResults
        
            #RMS Level
            #Get the object for the LevelVsFrequency results
            LevelVsFrequencyXYResults = Results[MeasurementResultType.LevelVsFrequency]
            
            RMSLevelXAxis_Data = LevelVsFrequencyXYResults.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
            print RMSLevelXAxis_Data
            RMSLevelYAxis_Ch1_LevelData = LevelVsFrequencyXYResults.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
            print RMSLevelYAxis_Ch1_LevelData
            RMSLevelYAxis_Ch2_LevelData = LevelVsFrequencyXYResults.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1)
            print RMSLevelYAxis_Ch2_LevelData

            #RMS Level Smoothed Derived Result
            #Get the object for the RMS Level -> Smooth results
            SmoothedXYResults = Results["RMS Level -> Smooth"]

            SmoothXValues = SmoothedXYResults.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
            print SmoothXValues
            SmoothYValues_Ch1 = SmoothedXYResults.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
            print SmoothYValues_Ch1
            SmoothYValues_Ch2 = SmoothedXYResults.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1)
            print SmoothYValues_Ch2
            
            #Deviation
            DeviationXValues = Results["Deviation (20.0000 Hz - 20.0000 kHz)"].GetMeterValues()
            print DeviationXValues
            
            #Alternate approach using For statement to get the data
            for Result in Results.Item:
                #RMS Level
                if Result.Name == "RMS Level":
                    print "RMS Level"
                    RMSLevelXAxis_Data = Result.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
                    RMSLevelYAxis_Ch1_LevelData = Result.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
                    RMSLevelYAxis_Ch2_LevelData = Result.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1)
                #RMS Level -> Smooth
                if Result.Name == "RMS Level -> Smooth":
                    print "RMS Level -> Smooth"
                    SmoothXValues = Result.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
                    SmoothYValues_Ch1 = Result.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
                    SmoothYValues_Ch2 = Result.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1)
                #Deviation
                if Result.Name == "Deviation (20.0000 Hz - 20.0000 kHz)":
                    print "Deviation (20.0000 Hz - 20.0000 kHz)"
                    DeviationXValues = Result.GetMeterValues()
                    break
                                 
    def APxBenchMode(self, sender, args):
        print "Bench Mode Automation"   
        
        APx = APx500_Application(APxOperatingMode.BenchMode)
        
        #Configure Signal Path Setup Input/Output
        print "Configure Signal Path Setup Input/Output"
        APx.BenchMode.Setup.OutputConnector.Type = OutputConnectorType.AnalogBalanced
        APx.BenchMode.Setup.AnalogOutput.ChannelCount = 2
        APx.BenchMode.Setup.AnalogOutput.BalancedSourceImpedance = AnalogBalancedSourceImpedance.SourceImpedance_40
    
        APx.BenchMode.Setup.InputConnector.Type = InputConnectorType.AnalogBalanced
        APx.BenchMode.Setup.AnalogInput.ChannelCount = 2
        APx.BenchMode.Setup.AnalogInput.SetTermination(InputChannelIndex.Ch1, AnalogInputTermination.InputTermination_Bal_200k)
        APx.BenchMode.Setup.AnalogInput.UseHighPerformanceSineAnalyzer = True
    
        #Configure Signal Path Setup Filters
        print "Configure Signal Path Setup Filters"
        APx.BenchMode.Setup.HighpassFilter = HighpassFilterMode.DC
        APx.BenchMode.Setup.LowpassFilterAnalog = LowpassFilterModeAnalog.Elliptic
        APx.BenchMode.Setup.LowpassFilterFrequencyAnalog = 20000
        APx.BenchMode.Setup.LowpassFilterAnalogBandwidth = AdcBandwidth.bw90k
        APx.BenchMode.Setup.WeightingFilter = SignalPathWeightingFilterType.wt_None
        APx.BenchMode.Setup.DeviceDelay.Value = 0
    
        #Configure Generator
        print "Configure Generator"
        APx.BenchMode.Generator.AutoOn = False
        APx.BenchMode.Generator.AnalogSineMode = AnalogSineGeneratorMode.HighPerformanceSineGenerator
        APx.BenchMode.Generator.Levels.SetValue(OutputChannelIndex.Ch1, 1.0)
        APx.BenchMode.Generator.Frequency.Value = 2000
        APx.BenchMode.Generator.On = True
    
        #Configure Analyzer
        print "Configure Analyzer"
        APx.BenchMode.Analyzer.BandpassFilterTuning = FilterTuningType.GeneratorFrequency
        APx.BenchMode.Analyzer.ThdNWeighting = SignalPathWeightingFilterType.wt_None
        APx.BenchMode.Analyzer.BandpassFilterTuning = FilterTuningType.GeneratorFrequency
        APx.BenchMode.Analyzer.ReferenceChannel = InputChannelIndex.Ch1
        APx.BenchMode.Analyzer.ImdMeterType = ImdAnalysisType.SmpteDin
    
        #Monitors/Meters
        #Add required Monitors/Meters
        APx.BenchMode.Meters.Show()
        APx.BenchMode.Meters.Enabled = False
    
        #Delete current Monitors/Meters  
        print "Delete current Monitors/Meters"
        MeterCount = APx.BenchMode.Meters.Count
        for meterIndex in range(MeterCount - 1, -1, -1):
            APx.BenchMode.Meters.Remove((meterIndex))
                       
        #Add desired Monitors/Meters
        print "Add desired Monitors/Meters"
        APx.BenchMode.Meters.Add(BenchModeMeterType.BandpassLevelMeter)
        APx.BenchMode.Meters.Add(BenchModeMeterType.DcLevelMeter)
        APx.BenchMode.Meters.Add(BenchModeMeterType.GainMeter)
        APx.BenchMode.Meters.Add(BenchModeMeterType.ThdNRatioMeter)
    
        #Configure settling for desired Monitors/Meters
        print "Configure settling for desired Monitors/Meters"
        APx.BenchMode.Analyzer.Settling.Enabled = True
        APx.BenchMode.Analyzer.Settling.ReadingTimeout = 5
        APx.BenchMode.Analyzer.Settling.TrackFirst = True
        
        APx.BenchMode.Analyzer.Settling[SettlingMeterType.RmsLevel].ResetToDefaults
        
        APx.BenchMode.Analyzer.Settling[SettlingMeterType.BandpassLevel][InputChannelIndex.Ch1].Algorithm = SettlingAlgorithmEx.Average
    
        ChannelSettlingParameters = APx.BenchMode.Analyzer.Settling[SettlingMeterType.BandpassLevel][InputChannelIndex.Ch1]    
        ChannelSettlingParameters.Algorithm = SettlingAlgorithmEx.None
        ChannelSettlingParameters.DelayTime = 0.031
        ChannelSettlingParameters.DelayCycles = 4
        ChannelSettlingParameters.SettlingTime = 0.251
        ChannelSettlingParameters.SettlingCycles = 4
        ChannelSettlingParameters.Points = 4
        ChannelSettlingParameters.Tolerance = 4
        ChannelSettlingParameters.Floor = 0.00000002
    
        APx.BenchMode.Analyzer.Settling.ResetToDefaults()
    
        APx.BenchMode.Meters.Enabled = True
        
        #Get Settled readings from meters all at the same time.
        #Create an array of settling meter types
        SettledMeterTypes = (SettlingMeterType.BandpassLevel, SettlingMeterType.DcLevel, SettlingMeterType.Gain, SettlingMeterType.ThdNRatio)   
        #Pass an array of meter types to get the ISettledResultCollection 
        MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettledMeterTypes)
        
        #Get reading for each meter type
        BandpassValues = MeterReadings[SettlingMeterType.BandpassLevel].GetValues("Vrms")
        print BandpassValues
        DCLevelValues = MeterReadings[SettlingMeterType.DcLevel].GetValues("V")
        print DCLevelValues
        GainValues = MeterReadings[SettlingMeterType.Gain].GetValues("dB")
        print GainValues
        ThdNValues = MeterReadings[SettlingMeterType.ThdNRatio].GetValues("dB")
        print ThdNValues
        
        #Get Settled readings from meters one at a time.
        MeterReadings = APx.BenchMode.GetSettledMeterReadings(Array[SettlingMeterType]([SettlingMeterType.BandpassLevel]))
        BandpassValues = MeterReadings[0].GetValues("Vrms")
        print BandpassValues
        MeterReadings = APx.BenchMode.GetSettledMeterReadings(Array[SettlingMeterType]([SettlingMeterType.DcLevel]))
        DCLevelValues = MeterReadings[0].GetValues("V")
        print DCLevelValues
        MeterReadings = APx.BenchMode.GetSettledMeterReadings(Array[SettlingMeterType]([SettlingMeterType.Gain]))
        GainValues = MeterReadings[0].GetValues("dB")
        print GainValues
        MeterReadings = APx.BenchMode.GetSettledMeterReadings(Array[SettlingMeterType]([SettlingMeterType.ThdNRatio]))
        ThdNValues = MeterReadings[0].GetValues("dB")
        print ThdNValues
            
        #Sweep measurement
        #Configure measurement settings
        print "Configure Sweep measurement settings"
        APx.BenchMode.Measurements.SteppedSweep.Show()
        APx.BenchMode.Measurements.SteppedSweep.Repeat = False
        APx.BenchMode.Measurements.SteppedSweep.Append = False
        APx.BenchMode.Measurements.SteppedSweep.Source = SweepSourceParameterType.GeneratorFrequency
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Start.Unit = "Hz"
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Start.Value = 20
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Stop.Value = 20000
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.StepType = SweepStepType.Log
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.NumberOfPoints = 31
     
        #Set Right axis data to THD+N Ratio
        print "Set Right axis data to THD+N Ratio"
        APx.BenchMode.SignalPathSettingsVisible = False
        APx.BenchMode.GeneratorAnalyzerSettingsVisible = False
        
        APx.BenchMode.Measurements.SteppedSweep.Graphs[0].Result.AsXYGraph().DefineRightAxisResult(MeasurementResultType.ThdNRatioVsFrequency)
        
        APx.BenchMode.Measurements.SteppedSweep.ReadingRate = SweepReadingRate.ReadingRate_32
        APx.BenchMode.Measurements.SteppedSweep.PreSweepDelay = 0.3
        
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.Source = NestedSweepParameterType.GeneratorLevel
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.Start.Value = 1
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.Stop.Value = 2
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.StepType = SweepStepType.Lin
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.NumberOfPoints = 2
                
        APx.BenchMode.Measurements.SteppedSweep.Start()
        print "Start Stepped Sweep measurement"
        
        #Wait for Sweep to finish
        while APx.BenchMode.Measurements.SteppedSweep.IsStarted:
            print ".",
            time.sleep(1)
        print "."
        
        #Get Sweep data from the two Sweeps
        SweepXYResults = APx.BenchMode.Measurements.SteppedSweep.Graphs["RMS Level"].Result.AsXYGraph()
        
        #Get X values from Sweep 1
        SweepXAxisValues = SweepXYResults.GetXValues(InputChannelIndex.Ch1, 0, "Hz", VerticalAxis.Left)
        print SweepXAxisValues
        #Get Y values from Sweep 1
        SweepAcq1YAxis_LevelValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 0, "Vrms", VerticalAxis.Left)
        print SweepAcq1YAxis_LevelValues
        SweepAcq1YAxis_THDNValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 0, "dB", VerticalAxis.Right)
        print SweepAcq1YAxis_THDNValues
        #Get Y values from Sweep 2
        SweepAcq2YAxis_LevelValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 1, "Vrms", VerticalAxis.Left)
        print SweepAcq2YAxis_LevelValues
        SweepAcq2YAxis_THDNValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 1, "dB", VerticalAxis.Right)
        print SweepAcq2YAxis_THDNValues
        
        #FFT measurement
        #Configure measurement settings
        print "Configure FFT measurement settings"
        APx.BenchMode.Measurements.Fft.Show()
        APx.BenchMode.Measurements.Fft.Repeat = False
        APx.BenchMode.Measurements.Fft.Append = False
        APx.BenchMode.Measurements.Fft.AcquisitionType = AcqLengthType.Auto
        APx.BenchMode.Measurements.Fft.FFTLength = FFTLength.FFT_96000
        APx.BenchMode.Measurements.Fft.WindowType = WindowType.BlackmanHarris
        APx.BenchMode.Measurements.Fft.TriggerType = TriggerType.PositiveZeroCrossing
        APx.BenchMode.Measurements.Fft.TriggerChannel = InputChannelIndex.Ch1
        APx.BenchMode.Measurements.Fft.AveragingType = AveragingType.Power
        APx.BenchMode.Measurements.Fft.Averages = 8
        APx.BenchMode.Measurements.Fft.DelayTimeInSeconds = 0.1
            
        APx.BenchMode.Measurements.Fft.Start()
        print "Start FFT measurement"

        #Wait for FFT to finish
        while APx.BenchMode.Measurements.Fft.IsStarted:
            print ".",
            time.sleep(1)
        print "."
        
        #Get all FFT data
        FFTXYResults = APx.BenchMode.Measurements.Fft.Graphs["FFT Spectrum"].Result.AsXYGraph()
        FFTXAxisValues = FFTXYResults.GetAllXValues(InputChannelIndex.Ch1, "Hz", VerticalAxis.Left)
        print FFTXAxisValues
        FFTYAxis_Ch1_LevelValues = FFTXYResults.GetAllYValues(InputChannelIndex.Ch1, "Vrms", VerticalAxis.Left)
        print FFTYAxis_Ch1_LevelValues
        FFTYAxis_Ch2_LevelValues = FFTXYResults.GetAllYValues(InputChannelIndex.Ch2, "Vrms", VerticalAxis.Left)
        print FFTYAxis_Ch2_LevelValues
        
        #Get all Scope data
        ScopeXYResults = APx.BenchMode.Measurements.Fft.Graphs["Scope"].Result.AsXYGraph()
        ScopeXAxisValues = ScopeXYResults.GetAllXValues(InputChannelIndex.Ch1, "s", VerticalAxis.Left)
        print ScopeXAxisValues
        ScopeYAxis_Ch1_LevelValues = ScopeXYResults.GetAllYValues(InputChannelIndex.Ch1, "V", VerticalAxis.Left)
        print ScopeYAxis_Ch1_LevelValues
        ScopeYAxis_Ch2_LevelValues = ScopeXYResults.GetAllYValues(InputChannelIndex.Ch2, "V", VerticalAxis.Left)
        print ScopeYAxis_Ch2_LevelValues
      
form = Container()
Application.Run(form)