﻿Imports AudioPrecision.API
Public Class Form1
    'Create an APx object
    Private APx As New APx500()

    Public Sub New()
        Call CreateAPxObject()

        ' This call is required by the designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call.

    End Sub

    Private Sub CreateAPxObject()
        APx = New AudioPrecision.API.APx500
    End Sub

    Private Sub Button1_Click(sender As Object, e As EventArgs) Handles Button1.Click
        APx.Visible = True
    End Sub

    Private Sub Button2_Click(sender As Object, e As EventArgs) Handles Button2.Click
        Dim filename As String = "SampleProject.approjx"
        Dim directory As String = System.Environment.CurrentDirectory
        Dim fullpath As String = directory & "\" & filename

        APx.OperatingMode = APxOperatingMode.SequenceMode
        APx.OpenProject(fullpath)
        APx.Sequence.Run()

        'Get Settled results from Level and Gain measurement.
        Dim measurement As ISequenceMeasurement = APx.Sequence("Signal Path1")("Level and Gain")

        'Check to ensure settled results are available
        If measurement.HasSequenceResults Then
            'Get the object for the Level and Gain sequence results
            Dim Results As ISequenceResultCollection
            Results = measurement.SequenceResults

            'get the meter readings
            'RMS Level
            Dim RMSLevelValues As Double() = Results.Item(MeasurementResultType.LevelMeter).GetMeterValues()

            'Gain
            Dim GainValues As Double() = Results.Item(MeasurementResultType.GainMeter).GetMeterValues()

            'Peak Level
            Dim PeakLevelValues As Double() = Results.Item(MeasurementResultType.PeakLevelMeter).GetMeterValues()
        End If

        'Get X/Y results from Stepped Frequency Sweep
        measurement = APx.Sequence("Signal Path1")("Stepped Frequency Sweep")

        'Check the to ensure results are available
        If measurement.HasSequenceResults = True Then
            'Data Variables
            Dim RMSLevelXAxis_Data() As Double
            Dim RMSLevelYAxis_Ch1_LevelData() As Double
            Dim RMSLevelYAxis_Ch2_LevelData() As Double

            Dim SmoothXValues As Double()
            Dim SmoothYValues_Ch1 As Double()
            Dim SmoothYValues_Ch2 As Double()

            Dim DeviationXValues As Double()

            'Get the object for the Stepped Frequency Sweep sequence results
            Dim Results As ISequenceResultCollection
            Results = measurement.SequenceResults

            'RMS Level
            Dim LevelVsFrequencyXYResults As ISequenceResult
            'Get the object for the LevelVsFrequency results
            LevelVsFrequencyXYResults = Results.Item(MeasurementResultType.LevelVsFrequency)

            RMSLevelXAxis_Data = LevelVsFrequencyXYResults.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
            RMSLevelYAxis_Ch1_LevelData = LevelVsFrequencyXYResults.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
            RMSLevelYAxis_Ch2_LevelData = LevelVsFrequencyXYResults.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1)

            'RMS Level Smoothed Derived Result
            Dim SmoothedXYResults As ISequenceResult
            'Get the object for the RMS Level -> Smooth results
            SmoothedXYResults = Results.Item("RMS Level -> Smooth")

            SmoothXValues = SmoothedXYResults.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
            SmoothYValues_Ch1 = SmoothedXYResults.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
            SmoothYValues_Ch2 = SmoothedXYResults.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1)

            'Deviation
            DeviationXValues = Results("Deviation (20.0000 Hz - 20.0000 kHz)").GetMeterValues()

            'Alternate approach using For Each statement to get the data
            For Each Result As ISequenceResult In Results
                'RMS Level
                If Result.Name = "RMS Level" Then
                    RMSLevelXAxis_Data = Result.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
                    RMSLevelYAxis_Ch1_LevelData = Result.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
                    RMSLevelYAxis_Ch2_LevelData = Result.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1)
                End If
                'RMS Level -> Smooth
                If Result.ResultType.Equals(MeasurementResultType.Smooth) And Result.Name = "RMS Level -> Smooth" Then
                    SmoothXValues = Result.GetXValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
                    SmoothYValues_Ch1 = Result.GetYValues(InputChannelIndex.Ch1, VerticalAxis.Left, SourceDataType.Measured, 1)
                    SmoothYValues_Ch2 = Result.GetYValues(InputChannelIndex.Ch2, VerticalAxis.Left, SourceDataType.Measured, 1)
                End If
                'Deviation
                If Result.Name = "Deviation (20.0000 Hz - 20.0000 kHz)" Then
                    DeviationXValues = Result.GetMeterValues()
                End If
            Next
        End If
    End Sub

    Private Sub Button3_Click(sender As Object, e As EventArgs) Handles Button3.Click
        APx.OperatingMode = APxOperatingMode.BenchMode

        'Configure Signal Path Setup Input/Output
        APx.BenchMode.Setup.OutputConnector.Type = OutputConnectorType.AnalogBalanced
        APx.BenchMode.Setup.AnalogOutput.ChannelCount = 2
        APx.BenchMode.Setup.AnalogOutput.BalancedSourceImpedance = AnalogBalancedSourceImpedance.SourceImpedance_40

        APx.BenchMode.Setup.InputConnector.Type = InputConnectorType.AnalogBalanced
        APx.BenchMode.Setup.AnalogInput.ChannelCount = 2
        APx.BenchMode.Setup.AnalogInput.SetTermination(InputChannelIndex.Ch1, AnalogInputTermination.InputTermination_Bal_200k)
        APx.BenchMode.Setup.AnalogInput.UseHighPerformanceSineAnalyzer = True

        'Configure Signal Path Setup Filters
        APx.BenchMode.Setup.HighpassFilter = HighpassFilterMode.DC
        APx.BenchMode.Setup.LowpassFilterAnalog = LowpassFilterModeAnalog.Elliptic
        APx.BenchMode.Setup.LowpassFilterFrequencyAnalog = 20000
        APx.BenchMode.Setup.LowpassFilterAnalogBandwidth = AdcBandwidth.bw90k
        APx.BenchMode.Setup.WeightingFilter = SignalPathWeightingFilterType.wt_None
        APx.BenchMode.Setup.DeviceDelay.Value = 0

        'Configure Generator
        APx.BenchMode.Generator.AutoOn = False
        APx.BenchMode.Generator.AnalogSineMode = AnalogSineGeneratorMode.HighPerformanceSineGenerator
        APx.BenchMode.Generator.Levels.SetValue(OutputChannelIndex.Ch1, 1.0)
        APx.BenchMode.Generator.Frequency.Value = 2000
        APx.BenchMode.Generator.On = True

        'Configure Analyzer
        APx.BenchMode.Analyzer.BandpassFilterTuning = FilterTuningType.GeneratorFrequency
        APx.BenchMode.Analyzer.ThdNWeighting = SignalPathWeightingFilterType.wt_None
        APx.BenchMode.Analyzer.BandpassFilterTuning = FilterTuningType.GeneratorFrequency
        APx.BenchMode.Analyzer.ReferenceChannel = InputChannelIndex.Ch1
        APx.BenchMode.Analyzer.ImdMeterType = ImdAnalysisType.SmpteDin

        'Monitors/Meters
        'Add required Monitors/Meters
        APx.BenchMode.Meters.Show()
        APx.BenchMode.Meters.Enabled = False

        'Delete current Monitors/Meters  
        Dim MeterCount As Integer = APx.BenchMode.Meters.Count
        For meterIndex = MeterCount - 1 To 0 Step -1
            APx.BenchMode.Meters.Remove(meterIndex)
        Next

        'Add desired Monitors/Meters
        APx.BenchMode.Meters.Add(BenchModeMeterType.BandpassLevelMeter)
        APx.BenchMode.Meters.Add(BenchModeMeterType.DcLevelMeter)
        APx.BenchMode.Meters.Add(BenchModeMeterType.GainMeter)
        APx.BenchMode.Meters.Add(BenchModeMeterType.ThdNRatioMeter)

        'Configure settling For desired Monitors/Meters
        APx.BenchMode.Analyzer.Settling.Enabled = True
        APx.BenchMode.Analyzer.Settling.ReadingTimeout = 5
        APx.BenchMode.Analyzer.Settling.TrackFirst = True

        APx.BenchMode.Analyzer.Settling(SettlingMeterType.RmsLevel).ResetToDefaults()

        APx.BenchMode.Analyzer.Settling(SettlingMeterType.BandpassLevel)(InputChannelIndex.Ch1).Algorithm = SettlingAlgorithmEx.Average

        Dim ChannelSettlingParameters As ISettlingParametersEx
        ChannelSettlingParameters = APx.BenchMode.Analyzer.Settling(SettlingMeterType.BandpassLevel)(InputChannelIndex.Ch1)
        ChannelSettlingParameters.Algorithm = SettlingAlgorithmEx.None
        ChannelSettlingParameters.DelayTime = 0.031
        ChannelSettlingParameters.DelayCycles = 4
        ChannelSettlingParameters.SettlingTime = 0.251
        ChannelSettlingParameters.SettlingCycles = 4
        ChannelSettlingParameters.Points = 4
        ChannelSettlingParameters.Tolerance = 4
        ChannelSettlingParameters.Floor = 0.00000002

        APx.BenchMode.Analyzer.Settling.ResetToDefaults()

        APx.BenchMode.Meters.Enabled = True

        'Get Settled readings from meters all at the same time.
        'Create an array of settling meter types
        Dim SettledMeterTypes = New SettlingMeterType() {SettlingMeterType.BandpassLevel, SettlingMeterType.DcLevel, SettlingMeterType.Gain, SettlingMeterType.ThdNRatio}
        'Pass the array of meter types to get the ISettledResultCollection 
        Dim MeterReadings As ISettledResultCollection
        MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettledMeterTypes)

        'Get readings for each meter type
        Dim BandpassValues() As Double = MeterReadings(SettlingMeterType.BandpassLevel).GetValues("Vrms")
        Dim DCLevelValues() As Double = MeterReadings(SettlingMeterType.DcLevel).GetValues("V")
        Dim GainValues() As Double = MeterReadings(SettlingMeterType.Gain).GetValues("dB")
        Dim ThdNValues() As Double = MeterReadings(SettlingMeterType.ThdNRatio).GetValues("dB")

        'Get Settled readings from meters one at a time.
        MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettlingMeterType.BandpassLevel)
        BandpassValues = MeterReadings.Item(0).GetValues("Vrms")

        MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettlingMeterType.DcLevel)
        DCLevelValues = MeterReadings.Item(0).GetValues("V")

        MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettlingMeterType.Gain)
        GainValues = MeterReadings.Item(0).GetValues("dB")

        MeterReadings = APx.BenchMode.GetSettledMeterReadings(SettlingMeterType.ThdNRatio)
        ThdNValues = MeterReadings.Item(0).GetValues("dB")

        'Sweep measurement
        'Configure measurement settings
        APx.BenchMode.Measurements.SteppedSweep.Show()
        APx.BenchMode.Measurements.SteppedSweep.Repeat = False
        APx.BenchMode.Measurements.SteppedSweep.Append = False
        APx.BenchMode.Measurements.SteppedSweep.Source = SweepSourceParameterType.GeneratorFrequency
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Start.Unit = "Hz"
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Start.Value = 20
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.Stop.Value = 20000
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.StepType = SweepStepType.Log
        APx.BenchMode.Measurements.SteppedSweep.SourceParameters.NumberOfPoints = 31

        'Set Right axis data To THD+N Ratio
        APx.BenchMode.SignalPathSettingsVisible = False
        APx.BenchMode.GeneratorAnalyzerSettingsVisible = False

        APx.BenchMode.Measurements.SteppedSweep.Graphs(0).Result.AsXYGraph().DefineRightAxisResult(MeasurementResultType.ThdNRatioVsFrequency)

        APx.BenchMode.Measurements.SteppedSweep.ReadingRate = SweepReadingRate.ReadingRate_32
        APx.BenchMode.Measurements.SteppedSweep.PreSweepDelay = 0.3

        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.Source = NestedSweepParameterType.GeneratorLevel
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.Start.Value = 1
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.Stop.Value = 2
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.StepType = SweepStepType.Lin
        APx.BenchMode.Measurements.SteppedSweep.NestedSweep.SourceParameters.NumberOfPoints = 2

        APx.BenchMode.Measurements.SteppedSweep.Start()

        'Wait for the Sweep to finish
        Do While APx.BenchMode.Measurements.SteppedSweep.IsStarted = True
            System.Threading.Thread.Sleep(1000)
        Loop

        'Get Sweep data from the two Sweeps
        Dim SweepXYResults As IXYGraph
        SweepXYResults = APx.BenchMode.Measurements.SteppedSweep.Graphs.Item("RMS Level").Result.AsXYGraph()

        Dim XAxisValues() As Double
        Dim Acq1YAxis_LevelValues() As Double
        Dim Acq1YAxis_THDNValues() As Double

        Dim Acq2YAxis_LevelValues() As Double
        Dim Acq2YAxis_THDNValues() As Double

        'Get X values from Sweep 1
        XAxisValues = SweepXYResults.GetXValues(InputChannelIndex.Ch1, 0, "Hz", VerticalAxis.Left)

        'Get Y values from Sweep 1
        Acq1YAxis_LevelValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 0, "Vrms", VerticalAxis.Left)
        Acq1YAxis_THDNValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 0, "dB", VerticalAxis.Right)
        'Get Y values from Sweep 2
        Acq2YAxis_LevelValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 1, "Vrms", VerticalAxis.Left)
        Acq2YAxis_THDNValues = SweepXYResults.GetYValues(InputChannelIndex.Ch1, 1, "dB", VerticalAxis.Right)

        'FFT measurement
        'Configure measurement settings
        APx.BenchMode.Measurements.Fft.Show()
        APx.BenchMode.Measurements.Fft.Repeat = False
        APx.BenchMode.Measurements.Fft.Append = False
        APx.BenchMode.Measurements.Fft.AcquisitionType = AcqLengthType.Auto
        APx.BenchMode.Measurements.Fft.FFTLength = FFTLength.FFT_96000
        APx.BenchMode.Measurements.Fft.WindowType = WindowType.BlackmanHarris
        APx.BenchMode.Measurements.Fft.TriggerType = TriggerType.PositiveZeroCrossing
        APx.BenchMode.Measurements.Fft.TriggerChannel = InputChannelIndex.Ch1
        APx.BenchMode.Measurements.Fft.AveragingType = AveragingType.Power
        APx.BenchMode.Measurements.Fft.Averages = 8
        APx.BenchMode.Measurements.Fft.DelayTimeInSeconds = 0.1

        APx.BenchMode.Measurements.Fft.Start()

        'Wait For FFT To finish
        Do While APx.BenchMode.Measurements.Fft.IsStarted = True
            System.Threading.Thread.Sleep(1000)
        Loop

        'Get all FFT data
        Dim FFTXYResults As IXYGraph
        Dim FFTXAxisValues() As Double
        Dim FFTYAxis_Ch1_LevelValues() As Double
        Dim FFTYAxis_Ch2_LevelValues() As Double

        FFTXYResults = APx.BenchMode.Measurements.Fft.Graphs.Item("FFT Spectrum").Result.AsXYGraph()

        FFTXAxisValues = FFTXYResults.GetAllXValues(InputChannelIndex.Ch1, "Hz", VerticalAxis.Left)
        FFTYAxis_Ch1_LevelValues = FFTXYResults.GetAllYValues(InputChannelIndex.Ch1, "Vrms", VerticalAxis.Left)
        FFTYAxis_Ch2_LevelValues = FFTXYResults.GetAllYValues(InputChannelIndex.Ch2, "Vrms", VerticalAxis.Left)

        'Get all Scope data
        Dim ScopeXYResults As IXYGraph
        Dim ScopeXAxisValues() As Double
        Dim ScopeYAxis_Ch1_LevelValues() As Double
        Dim ScopeYAxis_Ch2_LevelValues() As Double

        ScopeXYResults = APx.BenchMode.Measurements.Fft.Graphs.Item("Scope").Result.AsXYGraph()

        ScopeXAxisValues = ScopeXYResults.GetAllXValues(InputChannelIndex.Ch1, "s", VerticalAxis.Left)
        ScopeYAxis_Ch1_LevelValues = ScopeXYResults.GetAllYValues(InputChannelIndex.Ch1, "V", VerticalAxis.Left)
        ScopeYAxis_Ch2_LevelValues = ScopeXYResults.GetAllYValues(InputChannelIndex.Ch2, "V", VerticalAxis.Left)
    End Sub
End Class
